@@ -17,6 +17,7 @@ from tempfile import mkdtemp
 from six import StringIO
 
 from pelican.contents import Article
+from pelican.readers import default_metadata
 from pelican.settings import DEFAULT_CONFIG
 
 __all__ = ['get_article', 'unittest', ]
@@ -55,11 +56,11 @@ def isplit(s, sep=None):
 
     """
     sep, hardsep = r'\s+' if sep is None else re.escape(sep), sep is not None
-    exp, pos, l = re.compile(sep), 0, len(s)
+    exp, pos, length = re.compile(sep), 0, len(s)
     while True:
         m = exp.search(s, pos)
         if not m:
-            if pos < l or hardsep:
+            if pos < length or hardsep:
                 #      ^ mimic "split()": ''.split() returns []
                 yield s[pos:]
             break
@@ -113,9 +114,10 @@ def mute(returns_output=False):
     return decorator
 
 
-def get_article(title, slug, content, lang, extra_metadata=None):
-    metadata = {'slug': slug, 'title': title, 'lang': lang}
-    if extra_metadata is not None:
+def get_article(title, content, **extra_metadata):
+    metadata = default_metadata(settings=DEFAULT_CONFIG)
+    metadata['title'] = title
+    if extra_metadata:
         metadata.update(extra_metadata)
     return Article(content, metadata=metadata)
 
@@ -173,11 +175,20 @@ def get_settings(**kwargs):
     return settings
 
 
+def get_context(settings=None, **kwargs):
+    context = settings.copy() if settings else {}
+    context['generated_content'] = {}
+    context['static_links'] = set()
+    context['static_content'] = {}
+    context.update(kwargs)
+    return context
+
+
 class LogCountHandler(BufferingHandler):
     """Capturing and counting logged messages."""
 
     def __init__(self, capacity=1000):
-        logging.handlers.BufferingHandler.__init__(self, capacity)
+        super(LogCountHandler, self).__init__(capacity)
 
     def count_logs(self, msg=None, level=None):
         return len([
