@@ -1,13 +1,16 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function, unicode_literals
 
-import collections
+try:
+    import collections.abc as collections
+except ImportError:
+    import collections
+
 import locale
 import logging
 import os
 import subprocess
 import sys
-
 from shutil import rmtree
 from tempfile import mkdtemp
 
@@ -109,7 +112,7 @@ class TestPelican(LoggedTestCase):
         self.assertDirsEqual(
             self.temp_path, os.path.join(OUTPUT_PATH, 'basic'))
         self.assertLogCountEqual(
-            count=3,
+            count=1,
             msg="Unable to find.*skipping url replacement",
             level=logging.WARNING)
 
@@ -208,6 +211,29 @@ class TestPelican(LoggedTestCase):
             msg="Writing .*",
             level=logging.INFO)
 
+    def test_cyclic_intersite_links_no_warnings(self):
+        settings = read_settings(path=None, override={
+            'PATH': os.path.join(CURRENT_DIR, 'cyclic_intersite_links'),
+            'OUTPUT_PATH': self.temp_path,
+            'CACHE_PATH': self.temp_cache,
+        })
+        pelican = Pelican(settings=settings)
+        mute(True)(pelican.run)()
+        # There are four different intersite links:
+        # - one pointing to the second article from first and third
+        # - one pointing to the first article from second and third
+        # - one pointing to the third article from first and second
+        # - one pointing to a nonexistent from each
+        # If everything goes well, only the warning about the nonexistent
+        # article should be printed. Only two articles are not sufficient,
+        # since the first will always have _context['generated_content'] empty
+        # (thus skipping the link resolving) and the second will always have it
+        # non-empty, containing the first, thus always succeeding.
+        self.assertLogCountEqual(
+            count=1,
+            msg="Unable to find '.*\\.rst', skipping url replacement.",
+            level=logging.WARNING)
+
     def test_md_extensions_deprecation(self):
         """Test that a warning is issued if MD_EXTENSIONS is used"""
         settings = read_settings(path=None, override={
@@ -222,3 +248,25 @@ class TestPelican(LoggedTestCase):
             count=1,
             msg="MD_EXTENSIONS is deprecated use MARKDOWN instead.",
             level=logging.WARNING)
+
+    def test_parse_errors(self):
+        # Verify that just an error is printed and the application doesn't
+        # abort, exit or something.
+        settings = read_settings(path=None, override={
+            'PATH': os.path.abspath(os.path.join(CURRENT_DIR, 'parse_error')),
+            'OUTPUT_PATH': self.temp_path,
+            'CACHE_PATH': self.temp_cache,
+        })
+        pelican = Pelican(settings=settings)
+        mute(True)(pelican.run)()
+        self.assertLogCountEqual(
+            count=1,
+            msg="Could not process .*parse_error.rst",
+            level=logging.ERROR)
+
+    def test_module_load(self):
+        """Test loading via python -m pelican --help displays the help"""
+        output = subprocess.check_output([
+            sys.executable, '-m', 'pelican', '--help'
+        ]).decode('ascii', 'replace')
+        assert 'usage:' in output
